/* This file is part of Chunky Monkey IRCD 
 *
 * Chunky Monkey IRCD is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Chunky Monkey IRCD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Chunky Monkey IRCD; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $Id: cmd_kick.c,v 1.9.4.9 2003/01/19 12:17:19 lloydy Exp $
 *
 */
#include "chunkyircd.h"
#include "channel.h"

CMIRCD_MODULE(1.03, CMD_CLIENT | CMD_SERVER);

/* Actual routine to kick a user */
static int kick_user(ServerInfo, ClientInfo, ClientInfo, ChannelInfo,
		     char *);

/* 
* Kick a user 
*/
int cmd_kick(ClientInfo c_ptr, ServerInfo s_ptr, Cmd cmd)
{
	ChannelInfo chan = NULL;
	ClientInfo kicker = NULL;
	ClientInfo victim = NULL;
	ChannelMember kicker_w = NULL;
	ChannelMember victim_w = NULL;
	char buffer[BUF_SIZE];

	/* Only Run checks if its a local client */
	if (c_ptr) {

		/* Setup the parameters we want */
		kicker = c_ptr;

		/* Have we got enough params? */
		if (!check_params(cmd, 2)) {
			/* Report not enought params */
			ircd_sprintf(buffer, BUF_SIZE,
				     "%s :Not enough parameters",
				     cmd->command);
			send_numeric(c_ptr, get_my_server(), 461, buffer);

			/* Nothing Nasty Happened */
			return 1;
		}

		/* Does the channel exist */
		chan = locate_channel(cmd->params[0]);

		if (chan == NULL) {
			/* Invalid channel */
			ircd_sprintf(buffer, BUF_SIZE,
				     "%s :No such channel",
				     cmd->params[0]);
			send_numeric(kicker, get_my_server(), 403, buffer);

			return 1;
		}

		/* Locate pointer to user wrapper structure */
		kicker_w= channel_is_client_on(kicker, chan);

		/* Get the pointer to the victim */
		victim = client_locate(cmd->params[1]);
		victim_w = channel_is_client_on(victim, chan);

		if (kicker_w== NULL && !(kicker->mode & MODE_IRCOP)) {
			/* Not on channel */
			ircd_sprintf(buffer, BUF_SIZE,
				     "%s :You're not on that channel",
				     chan->name);
			send_numeric(kicker, get_my_server(), 442, buffer);

			return 1;
		}

		if ((kicker_w->chan_mode & CHANUSR_OP) ||
		    (kicker->mode & MODE_IRCOP) ||
		    (kicker_w->chan_mode & CHANUSR_HALFOP)) {

			/* They have the correct permissions */
			kick_user(s_ptr, kicker, victim, chan, cmd->text);
		} else {
			/* They don't have channel ops */
			ircd_sprintf(buffer, BUF_SIZE,
				     "%s :Ha! You're no chan op!",
				     chan->name);
			send_numeric(kicker, get_my_server(), 482, buffer);

			return 1;
		}
	} else {
		/* Come from a remote server */
		if (cmd->origin) {
			chan = locate_channel(cmd->params[0]);
			kicker = client_locate(cmd->origin);
			if (kicker && chan) {
				
			}
		}
	}

	/* Return */
	return 1;
}

/* Actually kick users from the channel */
/* All checks been done */
static int kick_user(ServerInfo s_ptr,
		     ClientInfo kicker,
		     ClientInfo victim, ChannelInfo chan, char *reason)
{
	char buffer[BUF_SIZE];
	int i;
	ChannelMember kicker_w = NULL;
	ChannelMember victim_w = NULL;

	/* Get Channel Member structures */
	victim_w = channel_is_client_on(victim, chan);
	kicker_w = channel_is_client_on(kicker, chan);


	/* There are many kicking situations */

	if (victim_w && (	/* Always need victim on the channel */
				/* #1 - Channel Op kicking anyone else */
				(kicker_w
				 && (kicker_w->chan_mode & CHANUSR_OP)) ||
				/* #2 - IRC Op Kicking anyone */
				(kicker && (kicker->mode & MODE_IRCOP)) ||
				/* #3 - Channel Half-Op kicking anyone but an Op */
				(kicker_w
				 && (kicker_w->chan_mode && CHANUSR_HALFOP)
				 && !(victim_w->chan_mode && CHANUSR_OP))
	    )) {

		ircd_sprintf(buffer, BUF_SIZE, "KICK %s %s :%s",
			     chan->name, victim->nick,
			     reason ? reason : "");
		if (kicker && kicker->fd != -1)
			send_client_client(kicker, kicker, buffer);
		send_to_local_channel(kicker, chan, buffer);
		servers_relay(s_ptr, kicker->nick, buffer);

		/* Kick them */
		channel_rem_client(victim, chan);
		client_rem_chan(victim, chan);

		/* Close channel if needed */
		channel_needclose(chan);
	}

	return 0;
}
