/* This file is part of Chunky Monkey IRCD 
 *
 * Chunky Monkey IRCD is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Chunky Monkey IRCD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Chunky Monkey IRCD; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $Id: cmd_server.c,v 1.11.2.1.2.7 2003/01/19 12:17:19 lloydy Exp $
 *
 */
#include "modules.h"

CMIRCD_MODULE(1.02, CMD_SERVER);

int cmd_server(ClientInfo c_ptr, ServerInfo s_ptr, Cmd cmd)
{

	/* When i get a SERVER from an already registered server, I should have */
	/* :server_name SERVER new_server hops token :new_server info */
	/* so for struct serverinfo {
	 *      name = cmd->params[0]
	 *      hops = atoi(cmd->params[1])
	 *      token = atoi(cmd->params[2])
	 *      via  = server_locate(cmd->origin)
	 *      fd   = -1
	 *      }; */

	ServerInfo n_ptr = NULL;
	char buffer[BUF_SIZE];

	/* Somebody using the SERVER over a client connection */
	if (c_ptr || !check_params(cmd, 3) || !cmd->text || !cmd->origin) {

		return 0;
	}

	/* Now copy stuff into a new server */
	n_ptr = ircd_malloc(sizeof(struct _serverinfo));

	if (!n_ptr) {
		force_quit(s_ptr->fd, "Memory Error in cmd_server");

		return 0;
	}

	/* Check to see if the server already exists */
	if (server_locate(cmd->params[0])) {
		/* This is nasty */
		log_msg(LOG_BASIC,
			"Dropping Server Connection to %s, attempt to introduce dup %s",
			s_ptr->name, cmd->params[0]);
		force_quit(s_ptr->fd,
			   "Potential Damage To Network Intergrity");

		return 0;
	}

	/* Fill out structure */
	n_ptr->fd = -1;
	n_ptr->via_ptr = server_locate(cmd->origin);
	n_ptr->hops = n_ptr->via_ptr->hops + 1;
	ircd_sprintf(n_ptr->info, 200, "%s", cmd->text);
	strncpy(n_ptr->name, cmd->params[0], SERVNAME_MAX);

	/* Add into the hash */
	if (!server_hash_add(n_ptr)) {

		return 0;
	}

	/* Update Counters */
	n_ptr->via_ptr->server_count++;

	/* log */
	log_msg(LOG_BASIC, "Added new server connection to %s (%s) via %s",
		n_ptr->name, n_ptr->info, n_ptr->via_ptr->name);
	/* Announce */
	ircd_sprintf(buffer, BUF_SIZE,
		     "New server connection to %s, via server %s",
		     n_ptr->name, n_ptr->via_ptr->name);
	clients_servnotice(buffer);

	/* Relay to all new servers */
	ircd_sprintf(buffer, BUF_SIZE, "SERVER %s %d 0 :%s", n_ptr->name,
		     n_ptr->hops + 1, n_ptr->info);
	servers_relay(s_ptr, s_ptr->name, buffer);


	return 1;
}
